@@ -18,29 +18,17 @@
 use arrow_array::{builder::Int32Builder, RecordBatch};
 use arrow_schema::{DataType, Field, Schema};
 use criterion::{criterion_group, criterion_main, Criterion};
-use datafusion_comet_spark_expr::{Cast, EvalMode};
+use datafusion_comet_spark_expr::{Cast, EvalMode, SparkCastOptions};
 use datafusion_physical_expr::{expressions::Column, PhysicalExpr};
 use std::sync::Arc;
 
 fn criterion_benchmark(c: &mut Criterion) {
     let batch = create_int32_batch();
     let expr = Arc::new(Column::new("a", 0));
-    let timezone = "".to_string();
-    let cast_i32_to_i8 = Cast::new(
-        expr.clone(),
-        DataType::Int8,
-        EvalMode::Legacy,
-        timezone.clone(),
-        false,
-    );
-    let cast_i32_to_i16 = Cast::new(
-        expr.clone(),
-        DataType::Int16,
-        EvalMode::Legacy,
-        timezone.clone(),
-        false,
-    );
-    let cast_i32_to_i64 = Cast::new(expr, DataType::Int64, EvalMode::Legacy, timezone, false);
+    let spark_cast_options = SparkCastOptions::new_without_timezone(EvalMode::Legacy, false);
+    let cast_i32_to_i8 = Cast::new(expr.clone(), DataType::Int8, spark_cast_options.clone());
+    let cast_i32_to_i16 = Cast::new(expr.clone(), DataType::Int16, spark_cast_options.clone());
+    let cast_i32_to_i64 = Cast::new(expr, DataType::Int64, spark_cast_options);
 
     let mut group = c.benchmark_group("cast_int_to_int");
     group.bench_function("cast_i32_to_i8", |b| {
