@@ -872,6 +872,13 @@ fn cast_array(
     let array = array_with_timezone(array, cast_options.timezone.clone(), Some(to_type))?;
     let from_type = array.data_type().clone();
 
+    let native_cast_options: CastOptions = CastOptions {
+        safe: !matches!(cast_options.eval_mode, EvalMode::Ansi), // take safe mode from cast_options passed
+        format_options: FormatOptions::new()
+            .with_timestamp_tz_format(TIMESTAMP_FORMAT)
+            .with_timestamp_format(TIMESTAMP_FORMAT),
+    };
+
     let array = match &from_type {
         Dictionary(key_type, value_type)
             if key_type.as_ref() == &Int32
@@ -963,7 +970,7 @@ fn cast_array(
             || is_datafusion_spark_compatible(from_type, to_type, cast_options.allow_incompat) =>
         {
             // use DataFusion cast only when we know that it is compatible with Spark
-            Ok(cast_with_options(&array, to_type, &CAST_OPTIONS)?)
+            Ok(cast_with_options(&array, to_type, &native_cast_options)?)
         }
         _ => {
             // we should never reach this code because the Scala code should be checking
