@@ -1352,15 +1352,11 @@ object CometSparkSessionExtensions extends Logging {
     org.apache.spark.SPARK_VERSION >= "4.0"
   }
 
-  def isComplexTypeReaderEnabled(conf: SQLConf): Boolean = {
+  def usingDataFusionParquetExec(conf: SQLConf): Boolean = {
     CometConf.COMET_NATIVE_SCAN_IMPL.get(conf) == CometConf.SCAN_NATIVE_ICEBERG_COMPAT ||
     CometConf.COMET_NATIVE_SCAN_IMPL.get(conf) == CometConf.SCAN_NATIVE_DATAFUSION
   }
 
-  def usingDataFusionParquetReader(conf: SQLConf): Boolean = {
-    isComplexTypeReaderEnabled(conf) && !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)
-  }
-
   /** Calculates required memory overhead in MB per executor process for Comet. */
   def getCometMemoryOverheadInMiB(sparkConf: SparkConf): Long = {
     val baseMemoryMiB = if (cometUnifiedMemoryManagerEnabled(sparkConf)) {
@@ -1413,14 +1409,13 @@ object CometSparkSessionExtensions extends Logging {
     }
   }
 
-  def cometUnifiedMemoryManagerEnabled(sparkConf: SparkConf): Boolean = {
-    sparkConf.getBoolean("spark.memory.offHeap.enabled", false)
+  /** Calculates Comet shuffle memory size in MB */
+  def getCometShuffleMemorySizeInMiB(sparkConf: SparkConf, conf: SQLConf = SQLConf.get): Long = {
+    ByteUnit.BYTE.toMiB(getCometShuffleMemorySize(sparkConf, conf))
   }
 
-  def cometShuffleUnifiedMemoryManagerInTestEnabled(sparkConf: SparkConf): Boolean = {
-    sparkConf.getBoolean(
-      CometConf.COMET_COLUMNAR_SHUFFLE_UNIFIED_MEMORY_ALLOCATOR_IN_TEST.key,
-      CometConf.COMET_COLUMNAR_SHUFFLE_UNIFIED_MEMORY_ALLOCATOR_IN_TEST.defaultValue.get)
+  def cometUnifiedMemoryManagerEnabled(sparkConf: SparkConf): Boolean = {
+    sparkConf.getBoolean("spark.memory.offHeap.enabled", false)
   }
 
   /**
