@@ -19,15 +19,16 @@
 
 package org.apache.spark.sql.comet.shims
 
+
 import org.apache.hadoop.fs.Path
 
 import org.apache.spark.sql.SparkSession
 import org.apache.spark.sql.catalyst.InternalRow
-import org.apache.spark.sql.catalyst.expressions.{AttributeReference, Expression, FileSourceConstantMetadataAttribute, Literal}
+import org.apache.spark.sql.catalyst.expressions.{AttributeReference, Expression}
 import org.apache.spark.sql.errors.QueryExecutionErrors
-import org.apache.spark.sql.execution.datasources.parquet.ParquetOptions
+import org.apache.spark.sql.execution.{FileSourceScanExec, PartitionedFileUtil}
 import org.apache.spark.sql.execution.datasources._
-import org.apache.spark.sql.execution.{FileSourceScanExec, PartitionedFileUtil, ScalarSubquery}
+import org.apache.spark.sql.execution.datasources.parquet.ParquetOptions
 import org.apache.spark.sql.sources.Filter
 import org.apache.spark.sql.types.StructType
 
@@ -42,16 +43,14 @@ trait ShimCometScanExec {
       readFunction: PartitionedFile => Iterator[InternalRow],
       filePartitions: Seq[FilePartition],
       readSchema: StructType,
-      options: ParquetOptions): FileScanRDD = {
-    new FileScanRDD(
-      fsRelation.sparkSession,
-      readFunction,
-      filePartitions,
-      readSchema,
-      fileConstantMetadataColumns,
-      fsRelation.fileFormat.fileConstantMetadataExtractors,
-      options)
-  }
+      options: ParquetOptions): FileScanRDD = new FileScanRDD(
+    fsRelation.sparkSession,
+    readFunction,
+    filePartitions,
+    readSchema,
+    fileConstantMetadataColumns,
+    fsRelation.fileFormat.fileConstantMetadataExtractors,
+    options)
 
   protected def invalidBucketFile(path: String, sparkVersion: String): Throwable =
     QueryExecutionErrors.invalidBucketFile(path)
@@ -60,7 +59,7 @@ trait ShimCometScanExec {
   protected def isNeededForSchema(sparkSchema: StructType): Boolean = false
 
   protected def getPartitionedFile(f: FileStatusWithMetadata, p: PartitionDirectory): PartitionedFile =
-    PartitionedFileUtil.getPartitionedFile(f, p.values, 0, f.getLen)
+    PartitionedFileUtil.getPartitionedFile(f, p.values)
 
   protected def splitFiles(sparkSession: SparkSession,
                            file: FileStatusWithMetadata,
@@ -68,31 +67,10 @@ trait ShimCometScanExec {
                            isSplitable: Boolean,
                            maxSplitBytes: Long,
                            partitionValues: InternalRow): Seq[PartitionedFile] =
-    PartitionedFileUtil.splitFiles(file, isSplitable, maxSplitBytes, partitionValues)
+    PartitionedFileUtil.splitFiles(sparkSession, file, isSplitable, maxSplitBytes, partitionValues)
 
   protected def getPushedDownFilters(relation: HadoopFsRelation , dataFilters: Seq[Expression]):  Seq[Filter] = {
-    translateToV1Filters(relation, dataFilters, _.toLiteral)
-  }
-
-  // From Spark FileSourceScanLike
-  private def translateToV1Filters(relation: HadoopFsRelation,
-                                    dataFilters: Seq[Expression],
-                                    scalarSubqueryToLiteral: ScalarSubquery => Literal): Seq[Filter] = {
-    val scalarSubqueryReplaced = dataFilters.map(_.transform {
-      // Replace scalar subquery to literal so that `DataSourceStrategy.translateFilter` can
-      // support translating it.
-      case scalarSubquery: ScalarSubquery => scalarSubqueryToLiteral(scalarSubquery)
-    })
-
     val supportNestedPredicatePushdown = DataSourceUtils.supportNestedPredicatePushdown(relation)
-    // `dataFilters` should not include any constant metadata col filters
-    // because the metadata struct has been flatted in FileSourceStrategy
-    // and thus metadata col filters are invalid to be pushed down. Metadata that is generated
-    // during the scan can be used for filters.
-    scalarSubqueryReplaced.filterNot(_.references.exists {
-      case FileSourceConstantMetadataAttribute(_) => true
-      case _ => false
-    }).flatMap(DataSourceStrategy.translateFilter(_, supportNestedPredicatePushdown))
+    dataFilters.flatMap(DataSourceStrategy.translateFilter(_, supportNestedPredicatePushdown))
   }
-
 }
