@@ -25,12 +25,12 @@ import scala.util.Random
 import scala.util.matching.Regex
 
 import org.apache.hadoop.fs.Path
-import org.apache.spark.sql.{CometTestBase, DataFrame, SaveMode}
+import org.apache.spark.sql.{CometTestBase, DataFrame, Row, SaveMode}
 import org.apache.spark.sql.catalyst.expressions.Cast
 import org.apache.spark.sql.execution.adaptive.AdaptiveSparkPlanHelper
 import org.apache.spark.sql.functions.col
 import org.apache.spark.sql.internal.SQLConf
-import org.apache.spark.sql.types.{DataType, DataTypes, DecimalType}
+import org.apache.spark.sql.types.{DataType, DataTypes, DecimalType, StructField, StructType}
 
 import org.apache.comet.expressions.{CometCast, CometEvalMode, Compatible}
 
@@ -59,8 +59,9 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
 
   private val timestampPattern = "0123456789/:T" + whitespaceChars
 
-  lazy val usingDataFusionParquetReader: Boolean =
-    CometSparkSessionExtensions.usingDataFusionParquetReader(conf)
+  lazy val usingParquetExecWithIncompatTypes: Boolean =
+    CometSparkSessionExtensions.usingDataFusionParquetExec(conf) &&
+      !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)
 
   test("all valid cast combinations covered") {
     val names = testNames
@@ -151,63 +152,63 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     castTest(
       generateBytes(),
       DataTypes.BooleanType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to ShortType") {
     castTest(
       generateBytes(),
       DataTypes.ShortType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to IntegerType") {
     castTest(
       generateBytes(),
       DataTypes.IntegerType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to LongType") {
     castTest(
       generateBytes(),
       DataTypes.LongType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to FloatType") {
     castTest(
       generateBytes(),
       DataTypes.FloatType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to DoubleType") {
     castTest(
       generateBytes(),
       DataTypes.DoubleType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to DecimalType(10,2)") {
     castTest(
       generateBytes(),
       DataTypes.createDecimalType(10, 2),
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ByteType to StringType") {
     castTest(
       generateBytes(),
       DataTypes.StringType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   ignore("cast ByteType to BinaryType") {
     castTest(
       generateBytes(),
       DataTypes.BinaryType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   ignore("cast ByteType to TimestampType") {
@@ -215,7 +216,7 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     castTest(
       generateBytes(),
       DataTypes.TimestampType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   // CAST from ShortType
@@ -224,7 +225,7 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     castTest(
       generateShorts(),
       DataTypes.BooleanType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to ByteType") {
@@ -232,56 +233,56 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     castTest(
       generateShorts(),
       DataTypes.ByteType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to IntegerType") {
     castTest(
       generateShorts(),
       DataTypes.IntegerType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to LongType") {
     castTest(
       generateShorts(),
       DataTypes.LongType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to FloatType") {
     castTest(
       generateShorts(),
       DataTypes.FloatType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to DoubleType") {
     castTest(
       generateShorts(),
       DataTypes.DoubleType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to DecimalType(10,2)") {
     castTest(
       generateShorts(),
       DataTypes.createDecimalType(10, 2),
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   test("cast ShortType to StringType") {
     castTest(
       generateShorts(),
       DataTypes.StringType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   ignore("cast ShortType to BinaryType") {
     castTest(
       generateShorts(),
       DataTypes.BinaryType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   ignore("cast ShortType to TimestampType") {
@@ -289,7 +290,7 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
     castTest(
       generateShorts(),
       DataTypes.TimestampType,
-      hasIncompatibleType = usingDataFusionParquetReader)
+      hasIncompatibleType = usingParquetExecWithIncompatTypes)
   }
 
   // CAST from integer
@@ -899,6 +900,8 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("cast TimestampType to LongType") {
+    // https://github.com/apache/datafusion-comet/issues/1441
+    assume(!CometConf.isExperimentalNativeScan)
     assume(CometSparkSessionExtensions.isSpark33Plus)
     castTest(generateTimestampsExtended(), DataTypes.LongType)
   }
@@ -922,16 +925,22 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("cast TimestampType to StringType") {
+    // https://github.com/apache/datafusion-comet/issues/1441
+    assume(!CometConf.isExperimentalNativeScan)
     castTest(generateTimestamps(), DataTypes.StringType)
   }
 
   test("cast TimestampType to DateType") {
+    // https://github.com/apache/datafusion-comet/issues/1441
+    assume(!CometConf.isExperimentalNativeScan)
     castTest(generateTimestamps(), DataTypes.DateType)
   }
 
   // Complex Types
 
   test("cast StructType to StringType") {
+    // https://github.com/apache/datafusion-comet/issues/1441
+    assume(!CometConf.isExperimentalNativeScan)
     Seq(true, false).foreach { dictionaryEnabled =>
       withTempDir { dir =>
         val path = new Path(dir.toURI.toString, "test.parquet")
@@ -972,12 +981,15 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
   }
 
   test("cast between decimals with different precision and scale") {
-    // cast between default Decimal(38, 18) to Decimal(6,2)
-    val values = Seq(BigDecimal("12345.6789"), BigDecimal("9876.5432"), BigDecimal("123.4567"))
-    val df = withNulls(values)
-      .toDF("b")
-      .withColumn("a", col("b").cast(DecimalType(6, 2)))
-    checkSparkAnswer(df)
+    val rowData = Seq(
+      Row(BigDecimal("12345.6789")),
+      Row(BigDecimal("9876.5432")),
+      Row(BigDecimal("123.4567")))
+    val df = spark.createDataFrame(
+      spark.sparkContext.parallelize(rowData),
+      StructType(Seq(StructField("a", DataTypes.createDecimalType(10, 4)))))
+
+    castTest(df, DecimalType(6, 2))
   }
 
   test("cast between decimals with higher precision than source") {
@@ -1201,27 +1213,33 @@ class CometCastSuite extends CometTestBase with AdaptiveSparkPlanHelper {
               val cometMessage =
                 if (cometException.getCause != null) cometException.getCause.getMessage
                 else cometException.getMessage
-              if (CometSparkSessionExtensions.isSpark40Plus) {
-                // for Spark 4 we expect to sparkException carries the message
-                assert(
-                  sparkException.getMessage
-                    .replace(".WITH_SUGGESTION] ", "]")
-                    .startsWith(cometMessage))
-              } else if (CometSparkSessionExtensions.isSpark34Plus) {
-                // for Spark 3.4 we expect to reproduce the error message exactly
-                assert(cometMessage == sparkMessage)
+              // this if branch should only check decimal to decimal cast and errors when output precision, scale causes overflow.
+              if (df.schema("a").dataType.typeName.contains("decimal") && toType.typeName
+                  .contains("decimal") && sparkMessage.contains("cannot be represented as")) {
+                assert(cometMessage.contains("too large to store"))
               } else {
-                // for Spark 3.3 we just need to strip the prefix from the Comet message
-                // before comparing
-                val cometMessageModified = cometMessage
-                  .replace("[CAST_INVALID_INPUT] ", "")
-                  .replace("[CAST_OVERFLOW] ", "")
-                  .replace("[NUMERIC_VALUE_OUT_OF_RANGE] ", "")
-
-                if (sparkMessage.contains("cannot be represented as")) {
-                  assert(cometMessage.contains("cannot be represented as"))
+                if (CometSparkSessionExtensions.isSpark40Plus) {
+                  // for Spark 4 we expect to sparkException carries the message
+                  assert(
+                    sparkException.getMessage
+                      .replace(".WITH_SUGGESTION] ", "]")
+                      .startsWith(cometMessage))
+                } else if (CometSparkSessionExtensions.isSpark34Plus) {
+                  // for Spark 3.4 we expect to reproduce the error message exactly
+                  assert(cometMessage == sparkMessage)
                 } else {
-                  assert(cometMessageModified == sparkMessage)
+                  // for Spark 3.3 we just need to strip the prefix from the Comet message
+                  // before comparing
+                  val cometMessageModified = cometMessage
+                    .replace("[CAST_INVALID_INPUT] ", "")
+                    .replace("[CAST_OVERFLOW] ", "")
+                    .replace("[NUMERIC_VALUE_OUT_OF_RANGE] ", "")
+
+                  if (sparkMessage.contains("cannot be represented as")) {
+                    assert(cometMessage.contains("cannot be represented as"))
+                  } else {
+                    assert(cometMessageModified == sparkMessage)
+                  }
                 }
               }
           }
