@@ -19,6 +19,8 @@
 
 package org.apache.spark.sql
 
+import java.util.concurrent.atomic.AtomicInteger
+
 import scala.concurrent.duration._
 import scala.reflect.ClassTag
 import scala.reflect.runtime.universe.TypeTag
@@ -80,6 +82,7 @@ abstract class CometTestBase
     conf.set(CometConf.COMET_SHUFFLE_FALLBACK_TO_COLUMNAR.key, "true")
     conf.set(CometConf.COMET_SPARK_TO_ARROW_ENABLED.key, "true")
     conf.set(CometConf.COMET_NATIVE_SCAN_ENABLED.key, "true")
+    conf.set(CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.key, "true")
     conf.set(CometConf.COMET_MEMORY_OVERHEAD.key, "2g")
     conf.set(CometConf.COMET_EXEC_SORT_MERGE_JOIN_WITH_JOIN_FILTER_ENABLED.key, "true")
     conf
@@ -430,7 +433,8 @@ abstract class CometTestBase
   }
 
   def getPrimitiveTypesParquetSchema: String = {
-    if (CometSparkSessionExtensions.usingDataFusionParquetReader(conf)) {
+    if (CometSparkSessionExtensions.usingDataFusionParquetExec(conf) &&
+      !CometConf.COMET_SCAN_ALLOW_INCOMPATIBLE.get(conf)) {
       // Comet complex type reader has different behavior for uint_8, uint_16 types.
       // The issue stems from undefined behavior in the parquet spec and is tracked
       // here: https://github.com/apache/parquet-java/issues/3142
@@ -459,6 +463,7 @@ abstract class CometTestBase
          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
          |  optional INT32                    _20(DATE);
+         |  optional INT32                    _id;
          |}
         """.stripMargin
       } else {
@@ -484,6 +489,7 @@ abstract class CometTestBase
          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
          |  optional INT32                    _20(DATE);
+         |  optional INT32                    _id;
          |}
         """.stripMargin
       }
@@ -512,6 +518,7 @@ abstract class CometTestBase
          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
          |  optional INT32                    _20(DATE);
+         |  optional INT32                    _id;
          |}
         """.stripMargin
       } else {
@@ -537,6 +544,7 @@ abstract class CometTestBase
          |  optional INT64                    _18(TIMESTAMP(MILLIS,true));
          |  optional INT64                    _19(TIMESTAMP(MICROS,true));
          |  optional INT32                    _20(DATE);
+         |  optional INT32                    _id;
          |}
         """.stripMargin
       }
@@ -562,6 +570,8 @@ abstract class CometTestBase
       pageSize = pageSize,
       dictionaryPageSize = pageSize)
 
+    val idGenerator = new AtomicInteger(0)
+
     val rand = scala.util.Random
     val data = (begin until end).map { i =>
       if (rand.nextBoolean()) {
@@ -594,6 +604,7 @@ abstract class CometTestBase
           record.add(17, i.toLong)
           record.add(18, i.toLong)
           record.add(19, i)
+          record.add(20, idGenerator.getAndIncrement())
         case _ =>
       }
       writer.write(record)
@@ -621,6 +632,7 @@ abstract class CometTestBase
       record.add(17, i)
       record.add(18, i)
       record.add(19, i.toInt)
+      record.add(20, idGenerator.getAndIncrement())
       writer.write(record)
     }
 
