@@ -31,6 +31,7 @@ import org.apache.spark.sql.types._
 import org.apache.spark.sql.vectorized.ColumnVector
 
 import org.apache.comet.CometConf
+import org.apache.comet.CometConf.{SCAN_NATIVE_COMET, SCAN_NATIVE_DATAFUSION, SCAN_NATIVE_ICEBERG_COMPAT}
 import org.apache.comet.parquet.BatchReader
 
 /**
@@ -60,7 +61,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         sqlBenchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql(s"select $query from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql(s"select $query from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql(s"select $query from parquetV1Table").noop()
           }
         }
@@ -89,7 +108,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         sqlBenchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql("select sum(id) from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql("select sum(id) from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql("select sum(id) from parquetV1Table").noop()
           }
         }
@@ -197,7 +234,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         benchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql("select sum(c2) from parquetV1Table where c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql("select sum(c2) from parquetV1Table where c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql("select sum(c2) from parquetV1Table where c1 + 1 > 0").noop()
           }
         }
@@ -216,26 +271,44 @@ object CometReadBenchmark extends CometBenchmarkBase {
         prepareTable(
           dir,
           spark.sql(s"""
-             |WITH tmp
-             |  AS (SELECT RAND() r FROM $tbl)
-             |SELECT
-             |  CASE
-             |    WHEN r < 0.2 THEN 'aaa'
-             |    WHEN r < 0.4 THEN 'bbb'
-             |    WHEN r < 0.6 THEN 'ccc'
-             |    WHEN r < 0.8 THEN 'ddd'
-             |    ELSE 'eee'
-             |  END
-             |AS id
-             |FROM tmp
-             |""".stripMargin))
+                       |WITH tmp
+                       |  AS (SELECT RAND() r FROM $tbl)
+                       |SELECT
+                       |  CASE
+                       |    WHEN r < 0.2 THEN 'aaa'
+                       |    WHEN r < 0.4 THEN 'bbb'
+                       |    WHEN r < 0.6 THEN 'ccc'
+                       |    WHEN r < 0.8 THEN 'ddd'
+                       |    ELSE 'eee'
+                       |  END
+                       |AS id
+                       |FROM tmp
+                       |""".stripMargin))
 
         sqlBenchmark.addCase("SQL Parquet - Spark") { _ =>
           spark.sql("select sum(length(id)) from parquetV1Table").noop()
         }
 
         sqlBenchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql("select sum(length(id)) from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql("select sum(length(id)) from parquetV1Table").noop()
+          }
+        }
+
+        sqlBenchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql("select sum(length(id)) from parquetV1Table").noop()
           }
         }
@@ -266,7 +339,31 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         benchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark
+              .sql("select sum(length(c2)) from parquetV1Table where c1 is " +
+                "not NULL and c2 is not NULL")
+              .noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark
+              .sql("select sum(length(c2)) from parquetV1Table where c1 is " +
+                "not NULL and c2 is not NULL")
+              .noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark
               .sql("select sum(length(c2)) from parquetV1Table where c1 is " +
                 "not NULL and c2 is not NULL")
@@ -296,7 +393,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         benchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql(s"SELECT sum(c$middle) FROM parquetV1Table").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql(s"SELECT sum(c$middle) FROM parquetV1Table").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql(s"SELECT sum(c$middle) FROM parquetV1Table").noop()
           }
         }
@@ -327,7 +442,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         benchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
           }
         }
@@ -358,7 +491,25 @@ object CometReadBenchmark extends CometBenchmarkBase {
         }
 
         benchmark.addCase("SQL Parquet - Comet") { _ =>
-          withSQLConf(CometConf.COMET_ENABLED.key -> "true") {
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_COMET) {
+            spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native DataFusion") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_DATAFUSION) {
+            spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
+          }
+        }
+
+        benchmark.addCase("SQL Parquet - Comet Native Iceberg Compat") { _ =>
+          withSQLConf(
+            CometConf.COMET_ENABLED.key -> "true",
+            CometConf.COMET_NATIVE_SCAN_IMPL.key -> SCAN_NATIVE_ICEBERG_COMPAT) {
             spark.sql("SELECT * FROM parquetV1Table WHERE c1 + 1 > 0").noop()
           }
         }
